#ifndef main_h
#define main_h


#include <Arduino.h>

#include <ssdpAWS.h>              // SSDP for ESPAsyncWebServer
#include <DNSServer.h>            // Local DNS Server used for redirecting all requests to the configuration portal
#include <ESPAsyncWebServer.h>    // Local WebServer used to server the configuration portal
#include <ESPAsyncWiFiManager.h>  // https://github.com/alanswx/ESPAsyncWiFiManager

#define USE_FTP_SERVER
#define USE_OTA

#define use_telnet

#ifdef use_telnet
#include <TelnetSpy.h>
extern TelnetSpy LOG;
#define Log LOG
#else 
#define Log Serial
#endif

//**************************************************************************************************************************************************
//**                                                              Setting Camera type                                                             **
//**************************************************************************************************************************************************
//-------------------------------------------------------
// Global variables defined in Main.cpp
//-------------------------------------------------------
extern char esp_board[20];
extern char esp_name[20];
extern const char BUILD_MAIN[];
extern bool reboot;
extern long rebootdelay;

extern uint8_t ow_port_out;
extern uint8_t ow_port;
extern uint8_t ow_latch;
extern uint16_t ow_light;
extern char LastImg[32];

#define WROVER_KIT				0
#define ESP_EYE					1
#define M5STACK_PSRAM			2
#define M5STACK_WIDE			3
#define AI_THINKER				4

#define BOARD AI_THINKER

#if(BOARD == WROVER_KIT)
	#define PWDN_GPIO_NUM		-1
	#define RESET_GPIO_NUM		-1
	#define XCLK_GPIO_NUM		21
	#define SIOD_GPIO_NUM		26
	#define SIOC_GPIO_NUM		27
	#define Y9_GPIO_NUM			35
	#define Y8_GPIO_NUM			34
	#define Y7_GPIO_NUM			39
	#define Y6_GPIO_NUM			36
	#define Y5_GPIO_NUM			19
	#define Y4_GPIO_NUM			18
	#define Y3_GPIO_NUM			5
	#define Y2_GPIO_NUM			4
	#define VSYNC_GPIO_NUM		25
	#define HREF_GPIO_NUM		23
	#define PCLK_GPIO_NUM		22
	#define BUTTON_GPIO_NUM		12
	#define BUTTONLED_GPIO_NUM	13
	#define ON_LED_STATE  		HIGH
#elif(BOARD == ESP_EYE)
	#define PWDN_GPIO_NUM		-1
	#define RESET_GPIO_NUM		-1
	#define XCLK_GPIO_NUM		4
	#define SIOD_GPIO_NUM		18
	#define SIOC_GPIO_NUM		23
	#define Y9_GPIO_NUM			36
	#define Y8_GPIO_NUM			37
	#define Y7_GPIO_NUM			38
	#define Y6_GPIO_NUM			39
	#define Y5_GPIO_NUM			35
	#define Y4_GPIO_NUM			14
	#define Y3_GPIO_NUM			13
	#define Y2_GPIO_NUM			34
	#define VSYNC_GPIO_NUM		5
	#define HREF_GPIO_NUM		27
	#define PCLK_GPIO_NUM		25
	#define BUTTON_GPIO_NUM		15
	#define BUTTONLED_GPIO_NUM 	21
	#define ON_LED_STATE		HIGH
#elif(BOARD == M5STACK_PSRAM)
	#define PWDN_GPIO_NUM		-1
	#define RESET_GPIO_NUM		15
	#define XCLK_GPIO_NUM		27
	#define SIOD_GPIO_NUM		25
	#define SIOC_GPIO_NUM		23
	#define Y9_GPIO_NUM			19
	#define Y8_GPIO_NUM			36
	#define Y7_GPIO_NUM			18
	#define Y6_GPIO_NUM			39
	#define Y5_GPIO_NUM			5
	#define Y4_GPIO_NUM			34
	#define Y3_GPIO_NUM			35
	#define Y2_GPIO_NUM			32
	#define VSYNC_GPIO_NUM		22
	#define HREF_GPIO_NUM		26
	#define PCLK_GPIO_NUM		21
	#define BUTTON_GPIO_NUM		12
	#define BUTTONLED_GPIO_NUM	13
	#define ON_LED_STATE		HIGH
#elif(BOARD == M5STACK_WIDE)
	#define PWDN_GPIO_NUM		-1
	#define RESET_GPIO_NUM		15
	#define XCLK_GPIO_NUM		27
	#define SIOD_GPIO_NUM		22
	#define SIOC_GPIO_NUM		23
	#define Y9_GPIO_NUM			19
	#define Y8_GPIO_NUM			36
	#define Y7_GPIO_NUM			18
	#define Y6_GPIO_NUM			39
	#define Y5_GPIO_NUM			5
	#define Y4_GPIO_NUM			34
	#define Y3_GPIO_NUM			35
	#define Y2_GPIO_NUM			32
	#define VSYNC_GPIO_NUM		25
	#define HREF_GPIO_NUM		26
	#define PCLK_GPIO_NUM		21
	#define BUTTON_GPIO_NUM		12
	#define BUTTONLED_GPIO_NUM	13
	#define ON_LED_STATE		HIGH
#elif(BOARD == AI_THINKER)
	#define PWDN_GPIO_NUM		32
	#define RESET_GPIO_NUM		-1
	#define XCLK_GPIO_NUM		0
	#define SIOD_GPIO_NUM		26
	#define SIOC_GPIO_NUM		27
	#define Y9_GPIO_NUM			35
	#define Y8_GPIO_NUM			34
	#define Y7_GPIO_NUM			39
	#define Y6_GPIO_NUM			36
	#define Y5_GPIO_NUM			21
	#define Y4_GPIO_NUM			19
	#define Y3_GPIO_NUM			18
	#define Y2_GPIO_NUM			5
	#define VSYNC_GPIO_NUM		25
	#define HREF_GPIO_NUM		23
	#define PCLK_GPIO_NUM		22
	#define BUTTON_GPIO_NUM		12
	#define BUTTONLED_GPIO_NUM	13
	#define ON_LED_STATE		HIGH
#endif

#define BIT_TAST_BOARD	0x01    // 0 bei betaetigung
#define BIT_MOTION		0x02    // 1 bei bewegung
#define BIT_SND_BUSY	0x04    // 0 bei busy
#define BIT_TAST		0x08    // 0 bei betaetigung
#define BIT_OUT_LED		0x10    // 1 aktiv
#define BIT_OUT_2		0x20    // 1 aktiv
#define BIT_OUT_LIGHT	0x40    // 1 aktiv
#define BIT_OUT_RELAY	0x80    // 1 aktiv
//-------------------------------------------------------
// Global Functions

#define IS_SOUND_BUSY	(ow_port&BIT_SND_BUSY)
#define IS_OPEN_BUSY	(ow_port&BIT_OUT_RELAY)
#define IS_MOTION		(ow_port&BIT_MOTION)
#define IS_TAST			(ow_port&BIT_TAST)
#define IS_TAST_BOARD	(ow_port&BIT_TAST_BOARD)

void MakePhoto(void);

#endif
