#include "asc.h"
#include <base64.h>

void asc_null(AscConnection *asc)
{
	if(asc->url)
	{
		free(asc->url);
		asc->url=NULL;
	}
	asc->schema=NULL;
	asc->host=NULL;
	asc->path=NULL;
	asc->port=NULL;
	asc->query=NULL;
	asc->port_num=80;
}

void asc_reset(AscConnection *asc)
{
	if(asc->client)
	{
		asc->client->close(true);
		// on disconnect?
//		delete asc->client;
//		asc->client=NULL;
	}
	asc_null(asc);
}

AscConnection *asc_new(void)
{
	AscConnection *asc=NULL;

	asc=(AscConnection *)calloc(sizeof(AscConnection),1);
	if(asc)
	{
		asc->onConnect=NULL;
		asc->onDisconnect=NULL;
		asc->onError=NULL;
		asc->onData=NULL;
		asc->client=NULL;
		asc->headers=NULL;
		asc->ack_timeout=ASC_CON_TIMEOUT;
		asc->rx_timeout=ASC_RX_TIMEOUT;
		asc->url=NULL;
		asc_null(asc);
	}
	return asc;
}

void asc_del_headers(AscConnection *asc)
{
	if(asc->headers)
	{
		free(asc->headers);
		asc->headers=NULL;
	}
}

void asc_del(AscConnection *asc)
{
	asc_reset(asc);
	asc_del_headers(asc);
	free(asc);
}

void asc_addHeader(AscConnection *asc, char * header)
{
	if(asc->headers)
	{
		asc->headers=(char *)realloc(asc->headers,strlen(asc->headers)+strlen(header)+4);
		if(asc->headers)
		{
			strcat(asc->headers,header);
		}
	}
	else
	{
		asc->headers=(char *)malloc(strlen(header)+4);
		if(asc->headers)
		{
			strcpy(asc->headers,header);
		}
	}
}

void asc_auth_basic(AscConnection *asc, const char * user,const char * pw)
{
    if(user !=NULL && pw !=NULL)
	{
		if(strlen(user)>0 && strlen(pw)>0)
		{
			String auth = String(user);
			auth += ":";
			auth += String(pw);
			String ba = "Authorization:Basic ";
			ba += base64::encode(auth);
			ba += "\r\n";
			asc_addHeader(asc, (char *)(ba.c_str()));
		}
	}
}

bool asc_connect(AscConnection *asc,uint8_t method, const char* url)
{
	char *urlptr;
	char *last_urlptr;
	uint8_t state=STATE_SCHEMA;

	if(!asc)
	{
		return false;
	}

	asc_null(asc);

	if(method>ASC_METHODE_POST)
	{
		return false;
	}
	asc->method = method;
	asc->url=(char *)malloc(strlen(url) + 8);

	if(!asc->url)
	{
		return false;
	}
	strcpy(asc->url,url);
	urlptr=asc->url;
	last_urlptr=asc->url;
	// Find delimiter

	while(*urlptr)
	{
		bool do_default=true;
		bool do_end=false;

		switch(state)
		{
			case STATE_SCHEMA:
				if(strncmp(urlptr,"://",3)==0)
				{
					asc->schema=last_urlptr;
					*urlptr='\0';
					urlptr+=3;
					//Log.printf("Schema: %s\n\r",asc->schema);
					last_urlptr=urlptr;
					state=STATE_HOST;
					do_default=false;
				}
			break;
			case STATE_HOST:
				if(*urlptr == '/')
				{
					asc->host=last_urlptr;
					*urlptr='\0';
					urlptr++;
//					Log.printf("host: %s\n\r",asc->host);
					last_urlptr=urlptr;
					state=STATE_PATH;
					do_default=false;
				}
				else if(*urlptr == ':')
				{
					asc->host=last_urlptr;
					*urlptr='\0';
					urlptr++;
//					Log.printf("host: %s\n\r",asc->host);
					asc->port=urlptr;
					asc->port_num=atoi(urlptr);
					//Log.printf("port: %d\n\r",asc->port_num);
					state=STATE_PORT;
					do_default=false;
				}
			break;
			case STATE_PORT:
				if(*urlptr == '/')
				{
					*urlptr='\0';
					urlptr++;
					state=STATE_PATH;
					do_default=false;
				}
			break;
			case STATE_PATH:
				if(*urlptr == '?')
				{
					asc->path=last_urlptr;
					*urlptr='\0';
					urlptr++;
//					Log.printf("query path: %s\n\r",asc->path);
					asc->query=urlptr;
//					Log.printf("query: %s\n\r",asc->query);
					do_default=false;
					do_end=true;
					break;
				}
			break;
		}

		if(do_end)
		{
			break;
		}

		if(do_default)
		{
			urlptr++; 
			if(*urlptr=='\0')
			{
				if(!asc->host)
				{
					asc->host=last_urlptr;
//					Log.printf("set host: %s\n\r",asc->host);
					break;
				}
				else if(!asc->path)
				{
					asc->path=last_urlptr;
//					Log.printf("set path: %s\n\r",asc->path);
					break;
				}
			}
		}
	}

	if(asc->schema)
	{
		if(strcasecmp(asc->schema,"http")!=0)
		{
			// only http valid
			return false;
		}
	}
	if(!asc->host)
	{
		// no host
		return false;
	}
	if( !asc->client)
	{
		asc->client = new AsyncClient();
	}
	if(asc->client)
	{
	
		asc->client->setAckTimeout(asc->ack_timeout);
		asc->client->setRxTimeout(asc->rx_timeout);
		asc->client->onAck([](void* obj, AsyncClient* client, size_t len, uint32_t time){
			//AscConnection* oasc=(AscConnection*)(obj);
			//Log.println("OnAck " + String(millis()));
		}, asc);

		asc->client->onData([](void* obj, AsyncClient* client, void* data, size_t len)
		{
			AscConnection* oasc=(AscConnection*)(obj);
			//Log.println("OnData " + String(millis()) + " " +len);
			Log.write((uint8_t *)data,len);
			if(oasc->onData)
			{
				oasc->onData(oasc,data,len);
			}
		}, asc);

		asc->client->onConnect([](void *obj, AsyncClient *client)
		{
			AscConnection* oasc=(AscConnection*)(obj);
			Log.println("OnConnect " + String(millis()));
			if(oasc->onConnect)
			{
				oasc->onConnect(oasc);
			}
			String s=oasc->method ? "POST " : "GET ";
			s+="/";
			if(oasc->path) s+=oasc->path;
			if(oasc->query)
			{
				s+="?";
				s+=oasc->query;
			}
			s+=" HTTP/1.1\r\n";
			s+="host: "+String(oasc->host)+"\r\n";
			s+="user-agent: DoorBell32\r\n";
			s+="Connection: close\r\n";
			if(oasc->headers) s+=oasc->headers;
			s+="\r\n";
//			Log.println("OnConnect data send ["+s+"]");
			oasc->client->write(s.c_str());
		}, asc);

		asc->client->onDisconnect([](void *obj, AsyncClient* client)
		{
			AscConnection* oasc=(AscConnection*)(obj);
			Log.println("OnDisconnect " + String(millis()));
			if(oasc->onDisconnect)
			{
				oasc->onDisconnect(oasc);
			}
			if(oasc->client)
			{
				delete oasc->client;
				oasc->client=NULL;
			}
		}, asc);

		asc->client->onPoll([](void *obj, AsyncClient *client)
		{
			//AscConnection* oasc=(AscConnection*)(obj);
			//Log.println("OnPoll " + String(millis()));
		}, asc);

		asc->client->onError([](void *obj, AsyncClient *client, uint32_t error)
		{
			AscConnection* oasc=(AscConnection*)(obj);
			Log.println("OnError " + String(millis()) + " " + String((int32_t) error));
			if(oasc->onError)
			{
				oasc->onError(oasc,error);
			}
		}, asc);

#if ASYNC_TCP_SSL_ENABLED
//  if(_client->connect(host, port, secure)){
#else
//  if(_client->connect(host, port)){
#endif
		if( ! asc->client->connect(asc->host, asc->port_num))
		{
			Log.printf("!client.connect(%s, %d) failed\r\n", asc->host, asc->port_num);
            return false;
        }
	}
	else
	{
		return false;
	}
	return true;
}

void asc_init(void)
{

}

void asc_loop(void)
{

}

