#ifndef asc_h
#define asc_h

#include <Arduino.h>

#ifdef ESP32
#include <AsyncTCP.h>
#define _seize xSemaphoreTakeRecursive(threadLock,portMAX_DELAY)
#define _release xSemaphoreGiveRecursive(threadLock)
#else
#include <ESPAsyncTCP.h>
#define _seize 
#define _release 
#endif

#define use_telnet

#ifdef use_telnet
#include <TelnetSpy.h>
extern TelnetSpy LOG;
#define Log LOG
#else 
#define Log Serial
#endif

#define ASC_CON_TIMEOUT 		1000				// ms for connection timeout
#define ASC_RX_TIMEOUT 			2					// seconds for receive timeout
#define ASC_METHODE_GET			0
#define ASC_METHODE_POST		1

#define STATE_SCHEMA			0
#define STATE_HOST				1
#define STATE_PORT				2
#define STATE_PATH				3
#define STATE_QUERY				4
#define STATE_END				99

typedef void (*asc_on_cb_t)(struct ASC_CONNECTION *asc);
typedef void (*asc_onE_cb_t)(struct ASC_CONNECTION *asc,uint32_t error);
typedef void (*asc_onData_cb_t)(struct ASC_CONNECTION *asc,void* data, size_t len);

typedef struct ASC_CONNECTION
{
	char *url;
	char *schema;
	char *host;
	char *port;
	char *path;
	char *query;
	char *headers;
	AsyncClient *client;
	asc_on_cb_t onConnect;
	asc_onE_cb_t onError;
	asc_on_cb_t onDisconnect;
	asc_onData_cb_t onData;
	unsigned long ack_timeout;
	unsigned long rx_timeout;
	uint16_t port_num;
	uint8_t method;
	bool is_connected;
	bool have_send;
} AscConnection;

typedef struct ASC_LIST
{
	AscConnection *asc;
	void *next;
} AscList;

void asc_null(AscConnection *asc);
void asc_reset(AscConnection *asc);
AscConnection *asc_new(void);
void asc_del_headers(AscConnection *asc);
void asc_del(AscConnection *asc);
void asc_addHeader(AscConnection *asc, char * header);
void asc_auth_basic(AscConnection *asc, const char * user,const char * pw);
bool asc_connect(AscConnection *asc,uint8_t method, const char* url);
void asc_init(void);
void asc_loop(void);

#endif
