
#include <Arduino.h>
#include "Main.h"
#include "cfg.h"
#include "FS.h"                // SD Card ESP32
#include "SPIFFS.h"

Cfg cfg_global;

bool atob(char * p)
{
	if (strncasecmp(p,"true",4)==0)
	{
		return true;
	}
	return false;
}

bool cfg_LoadCfg(void)
{
	File file = SPIFFS.open("/cfg.json", "r");
	if (!file || file.isDirectory()) {
		return false;
	}
	String fileContent;
	while (file.available()) {
		fileContent += String((char)file.read());
		if (fileContent.length() > 5000) {
			Log.println(F("- cfg.json too large, assume it is corrupt"));
			fileContent = "";
			file.close();
			break;
		}
	}
	return cfg_parse((char *)(fileContent.c_str()));
}

int8_t cfg_SearchSnd(char *v)
{
	uint8_t i;
	for(i=0; i<CFG_SND_MAX;i++)
	{
		if(strcmp(v,cfg_global.sound[i].name.c_str())==0)
		{
			return i;
		}
	}
	return -1;
}

int8_t cfg_GetRing(void)
{
	uint8_t i;
	for(i=0; i<CFG_SND_MAX;i++)
	{
		if(cfg_global.sound[i].ring==true)
		{
			return i;
		}
	}
	return -1;
}


bool cfg_parse(char *conf)
{
	char *pt = 0;
	char buf1[128];
	uint8_t i;

	Log.println(F("Parse Config"));

	// set defaults
	cfg_global.flash_photo=true;
	cfg_global.flash_video=true;
	cfg_global.flash_dark=true;
	cfg_global.move_photo=true;
	cfg_global.move_light=true;
	cfg_global.move_dark=true;
	cfg_global.move_delay=20;
	cfg_global.move_call_on=false;
	cfg_global.move_url="";
	cfg_global.move_url_user="";
	cfg_global.move_url_pw="";
	cfg_global.light_dark=true;
	cfg_global.snd_on=true;
	cfg_global.snd_switch=true;
	cfg_global.opener_on=true;
	cfg_global.switch_photo=true;
	cfg_global.switch_call_on=false;
	cfg_global.switch_led_dark=false;
	cfg_global.switch_led_on=false;
	cfg_global.switch_led_flash=false;
	cfg_global.switch_led_times=5;
	cfg_global.switch_url="";
	cfg_global.switch_url_user="";
	cfg_global.switch_url_pw="";
	cfg_global.light_dark_val=200;
	cfg_global.light_bright_val=300;
	cfg_global.snd_vol=20;
	cfg_global.opener_time=2;
	cfg_global.motion_to=10;
	cfg_global.ring_to=10;
	cfg_global.sys_ntp="de.pool.ntp.org";
	cfg_global.sys_tz="CET-1CEST-2,M3.5.0/02:00:00,M10.5.0/03:00:00";
	cfg_global.un="admin";
	cfg_global.up="espadmin";

	for(i=0;i<CFG_SND_MAX;i++)
	{
		cfg_global.sound[i].name="";
	}

	pt=jsonbuf_parse(conf, (char *)"sound", NULL, sizeof(buf1)-1, -1);

	if(!pt)
	{
		Log.println(F("Error parse"));
		return false;
	}

	for(i=0;i<CFG_SND_MAX;i++)
	{
		if(jsonbuf_parse(pt, (char *)"name", buf1, sizeof(buf1)-1, i)) 
		{
			cfg_global.sound[i].name=String(buf1); 
			if(jsonbuf_parse(pt, (char *)"ring", buf1, sizeof(buf1)-1, i)) cfg_global.sound[i].ring=atob(buf1);
			if(jsonbuf_parse(pt, (char *)"open", buf1, sizeof(buf1)-1, i)) cfg_global.sound[i].open=atob(buf1);
			if(jsonbuf_parse(pt, (char *)"val", buf1, sizeof(buf1)-1, i)) cfg_global.sound[i].val=atoi(buf1);
			if(jsonbuf_parse(pt, (char *)"description", buf1, sizeof(buf1)-1, i)) cfg_global.sound[i].description=String(buf1);
		}
		else
		{
			break;
		}
	}

	if(jsonbuf_parse(conf, (char *)"flash_photo", buf1, sizeof(buf1)-1, -1)) cfg_global.flash_photo=atob(buf1);
	if(jsonbuf_parse(conf, (char *)"flash_video", buf1, sizeof(buf1)-1, -1)) cfg_global.flash_video=atob(buf1);
	if(jsonbuf_parse(conf, (char *)"flash_dark", buf1, sizeof(buf1)-1, -1)) cfg_global.flash_dark=atob(buf1);
	if(jsonbuf_parse(conf, (char *)"move_photo", buf1, sizeof(buf1)-1, -1)) cfg_global.move_photo=atob(buf1);
	if(jsonbuf_parse(conf, (char *)"move_light", buf1, sizeof(buf1)-1, -1)) cfg_global.move_light=atob(buf1);
	if(jsonbuf_parse(conf, (char *)"move_dark", buf1, sizeof(buf1)-1, -1)) cfg_global.move_dark=atob(buf1);
	if(jsonbuf_parse(conf, (char *)"move_delay", buf1, sizeof(buf1)-1, -1)) cfg_global.move_delay=atoi(buf1);
	if(jsonbuf_parse(conf, (char *)"move_call_on", buf1, sizeof(buf1)-1, -1)) cfg_global.move_call_on=atob(buf1);
	if(jsonbuf_parse(conf, (char *)"move_url", buf1, sizeof(buf1)-1, -1)) cfg_global.move_url=String(buf1);
	if(jsonbuf_parse(conf, (char *)"move_url_user", buf1, sizeof(buf1)-1, -1)) cfg_global.move_url_user=String(buf1);
	if(jsonbuf_parse(conf, (char *)"move_url_pw", buf1, sizeof(buf1)-1, -1)) cfg_global.move_url_pw=String(buf1);
	if(jsonbuf_parse(conf, (char *)"light_dark", buf1, sizeof(buf1)-1, -1)) cfg_global.light_dark=atob(buf1);
	if(jsonbuf_parse(conf, (char *)"snd_on", buf1, sizeof(buf1)-1, -1)) cfg_global.snd_on=atob(buf1);
	if(jsonbuf_parse(conf, (char *)"snd_switch", buf1, sizeof(buf1)-1, -1)) cfg_global.snd_switch=atob(buf1);
	if(jsonbuf_parse(conf, (char *)"opener_on", buf1, sizeof(buf1)-1, -1)) cfg_global.opener_on=atob(buf1);
	if(jsonbuf_parse(conf, (char *)"switch_photo", buf1, sizeof(buf1)-1, -1)) cfg_global.switch_photo=atob(buf1);
	if(jsonbuf_parse(conf, (char *)"switch_call_on", buf1, sizeof(buf1)-1, -1)) cfg_global.switch_call_on=atob(buf1);
	if(jsonbuf_parse(conf, (char *)"switch_led_dark", buf1, sizeof(buf1)-1, -1)) cfg_global.switch_led_dark=atob(buf1);
	if(jsonbuf_parse(conf, (char *)"switch_led_on", buf1, sizeof(buf1)-1, -1)) cfg_global.switch_led_on=atob(buf1);
	if(jsonbuf_parse(conf, (char *)"switch_led_flash", buf1, sizeof(buf1)-1, -1)) cfg_global.switch_led_flash=atob(buf1);
	if(jsonbuf_parse(conf, (char *)"switch_led_times", buf1, sizeof(buf1)-1, -1)) cfg_global.switch_led_times=atoi(buf1);
	if(jsonbuf_parse(conf, (char *)"switch_url", buf1, sizeof(buf1)-1, -1)) cfg_global.switch_url=String(buf1);
	if(jsonbuf_parse(conf, (char *)"switch_url_user", buf1, sizeof(buf1)-1, -1)) cfg_global.switch_url_user=String(buf1);
	if(jsonbuf_parse(conf, (char *)"switch_url_pw", buf1, sizeof(buf1)-1, -1)) cfg_global.switch_url_pw=String(buf1);
	if(jsonbuf_parse(conf, (char *)"light_dark_val", buf1, sizeof(buf1)-1, -1)) cfg_global.light_dark_val=atoi(buf1);
	if(jsonbuf_parse(conf, (char *)"light_bright_val", buf1, sizeof(buf1)-1, -1)) cfg_global.light_bright_val=atoi(buf1);
	if(jsonbuf_parse(conf, (char *)"snd_vol", buf1, sizeof(buf1)-1, -1)) cfg_global.snd_vol=atoi(buf1);
	if(jsonbuf_parse(conf, (char *)"opener_time", buf1, sizeof(buf1)-1, -1)) cfg_global.opener_time=atoi(buf1);
	if(jsonbuf_parse(conf, (char *)"motion_to", buf1, sizeof(buf1)-1, -1)) cfg_global.motion_to=atoi(buf1);
	if(jsonbuf_parse(conf, (char *)"ring_to", buf1, sizeof(buf1)-1, -1)) cfg_global.ring_to=atoi(buf1);
	if(jsonbuf_parse(conf, (char *)"sys_ntp", buf1, sizeof(buf1)-1, -1)) cfg_global.sys_ntp=String(buf1);
	if(jsonbuf_parse(conf, (char *)"sys_tz", buf1, sizeof(buf1)-1, -1)) cfg_global.sys_tz=String(buf1);
	if(jsonbuf_parse(conf, (char *)"un", buf1, sizeof(buf1)-1, -1)) cfg_global.un=String(buf1);
	if(jsonbuf_parse(conf, (char *)"up", buf1, sizeof(buf1)-1, -1)) cfg_global.up=String(buf1);
	return true;
}
