
#include <Arduino.h>
#include "dfplayer.h"

#define use_telnet

#ifdef use_telnet
#include <TelnetSpy.h>
extern TelnetSpy LOG;
#define Log LOG
#else 
#define Log Serial
#endif

#define DFPS Serial
//////////////////////////////////////////

uint8_t dfp_tx_buf[DFP_TX_LEN];
uint8_t dfp_rcv_buf[DFP_RX_LEN];
uint8_t dfp_rcv_index;
uint8_t dfp_resend_cnt;
uint8_t dfp_do_init;
unsigned long dfp_timeout;

bool dfp_busy=false;
bool dfp_do_to=false;

void dfp_init(void)
{
	dfp_resend_cnt=0;
	dfp_rcv_index=0;
	DFPS.flush();
	delay(20);
	dfp_do_init=2;
}

void dfp_loop(void)
{
	bool do_resend=false;
	if(dfp_do_init)
	{
		if(!dfp_busy)
		{
			dfp_setVolume(30);
			dfp_do_init--;
		}
	}

	while(DFPS.available())
	{
		uint8_t c=DFPS.read();
		Log.printf("0x%02X ",c);
		if((!dfp_rcv_index && c==DFP_Start_Byte) || dfp_rcv_index)
		{
			dfp_rcv_buf[dfp_rcv_index]=c;

			dfp_rcv_index++;
			if(dfp_rcv_index>=DFP_RX_LEN)
			{
				dfp_do_to=false;
				dfp_rcv_index=0;
				if(dfp_rcv_buf[1]==0xFF)
				{
					switch(dfp_rcv_buf[3])
					{
						case DFP_RES_ERROR:
							Log.println(F("Error DFPlayer"));
							if(dfp_resend_cnt<DFP_RESEND_NUM)
							{
								do_resend=true;
							}
							else
							{
								dfp_busy=false;
							}
							break;
						case DFP_RES_REPLY:
							Log.println(F("Play DFPlayer"));
							dfp_busy=true;
							break;
						case DFP_RES_STAY1:
						case DFP_RES_STAY2:
						case DFP_RES_STAY3:
							Log.println(F("Stay DFPlayer"));
							dfp_busy=false;
							break;
					}
				}
			}
		}
	}

#ifdef DFB_USE_TIMEOUT
	if(dfp_do_to)
	{
		if(millis()>dfp_timeout)
		{
			Log.println(F("Timeout DFPlayer"));
			if(dfp_resend_cnt<DFP_RESEND_NUM)
			{
				do_resend=true;
			}
			else
			{
				dfp_do_to=false;
				dfp_busy=false;
				dfp_rcv_index=0;
			}
		}
	}
#endif

	if(do_resend)
	{
		dfp_rcv_index=0;
		dfp_do_to=true;
		dfp_resend_cnt++;
		DFPS.write( dfp_tx_buf,DFP_TX_LEN);
		// retrigger timeout
		dfp_timeout=millis()+DFP_TIMEOUT_MS;
	}
}

void dfp_resend(void)
{
	dfp_execute_CMD(DFP_CMD_INC_VOL,0,0);
}

void dfp_inc_vol(void)
{
	dfp_execute_CMD(DFP_CMD_INC_VOL,0,0);
}

void dfp_dec_vol(void)
{
	dfp_execute_CMD(DFP_CMD_DEC_VOL,0,0);
}

void dfp_reset(void)
{
	dfp_execute_CMD(DFP_CMD_RES,0,0);
}

void dfp_pause(void)
{
	dfp_execute_CMD(DFP_CMD_PAUSE,0,0);
}

void dfp_play(void)
{
	dfp_execute_CMD(DFP_CMD_PLAY,0,1); 
}

void dfp_play(uint16_t s)
{
	dfp_busy=true;
	dfp_execute_CMD(DFP_CMD_MP3_FOLDER,s>>8,s&0xff); 
}

void dfp_playNext(void)
{
	dfp_execute_CMD(DFP_CMD_NEXT,0,1);
}

void dfp_playPrevious(void)
{
	dfp_execute_CMD(DFP_CMD_PREV,0,1);
}

void dfp_setVolume(int volume)
{
	dfp_execute_CMD(DFP_CMD_VOL, 0, volume); // Set the volume (0x00~0x30)
}

void dfp_execute_CMD(uint8_t CMD, uint8_t Par1, uint8_t Par2)
// Excecute the command and parameters
{
	// Calculate the checksum (2 bytes)
	int checksum = -(DFP_Version_Byte + DFP_Command_Length + CMD + DFP_Acknowledge + Par1 + Par2);
	// Build the command line in global buffer
	dfp_tx_buf[0]=DFP_Start_Byte;
	dfp_tx_buf[1]=DFP_Version_Byte;
	dfp_tx_buf[2]=DFP_Command_Length;
	dfp_tx_buf[3]=CMD;
	dfp_tx_buf[4]=DFP_Acknowledge;
	dfp_tx_buf[5]=Par1;
	dfp_tx_buf[6]=Par2;
	dfp_tx_buf[7]=highByte(checksum);
	dfp_tx_buf[8]=lowByte(checksum);
	dfp_tx_buf[9]=DFP_End_Byte;
	//Send the command line to the module
	DFPS.write( dfp_tx_buf,DFP_TX_LEN);
	dfp_resend_cnt=0;
	dfp_rcv_index=0;
	dfp_do_to=true;
	dfp_timeout=millis()+DFP_TIMEOUT_MS;
}

