#include <Arduino.h>
#include <Main.h>
#include <hw.h>
#include "MillisTimer.h"
#include "dfplayer.h"

MillisTimer Dark_timer = MillisTimer();
MillisTimer Relay_timer = MillisTimer();
MillisTimer Seconds_timer = MillisTimer();
MillisTimer Led_timer = MillisTimer();
MillisTimer Motion_timer = MillisTimer();
MillisTimer Ring_timer = MillisTimer();


bool work_dark=false;
bool work_do_flash_photo=false;
bool work_do_flash_video=false;
bool work_snd_busy=false;
bool work_do_open=false;
bool work_do_open_after_snd=false;
bool hw_motion=false;
bool hw_ring=false;
long seconds=0;
long light_timer = 0;
uint8_t led_count=0;

void Ring_tf(MillisTimer &mt)
{
	hw_ring=false;
}

void Motion_tf(MillisTimer &mt)
{
	hw_motion=false;
}

void Led_tf(MillisTimer &mt)
{
	hw_tgl_led();
//	if(led_count&1)
//	{
//		hw_do_led(false);
//	}
//	else
//	{
//		hw_do_led(true);
//	}
//	led_count++;
}

void Seconds_tf(MillisTimer &mt)
{
	seconds++;
}

void Relay_tf(MillisTimer &mt)
{
	ow_port_out&=~BIT_OUT_RELAY;
}

void Dark_tf(MillisTimer &mt)
{
	static bool last_dark=0;
	// detect darkness
	if(ow_light<cfg_global.light_dark_val)
	{
		work_dark=true;
	}
	else if(ow_light>cfg_global.light_bright_val)
	{
		work_dark=false;
	}

	// set flash-light to off
	work_do_flash_photo=false;
	work_do_flash_video=false;

	// set if flash in config
	if(cfg_global.flash_photo)
	{
		work_do_flash_photo=true;
	}

	if(cfg_global.flash_video)
	{
		work_do_flash_video=true;
	}

	// if flash only on darkness and it's bright, switch lights off 
	if(cfg_global.flash_dark && !work_dark)
	{
		work_do_flash_photo=false;
		work_do_flash_video=false;
	}
	
	if(cfg_global.light_dark && work_dark)
	{
		// switch light on
		// start or retrigger
		light_timer = seconds+cfg_global.move_delay;
		hw_do_light(true);
//		ow_port_out|=BIT_OUT_LIGHT;
	}
	if(last_dark!=work_dark)
	{
		if(cfg_global.switch_led_on || (cfg_global.switch_led_dark && work_dark))
		{
			hw_do_led(true);
		}
		else
		{
			hw_do_led(false);
		}
		last_dark=work_dark;
	}
}

void hw_open_door(void)
{
	if(cfg_global.opener_on)
	{
		ow_port_out|=BIT_OUT_RELAY;
		Relay_timer.setInterval(cfg_global.opener_time*1000);
		Relay_timer.reset();
		Relay_timer.start();
	}
}

void hw_do_light(bool v)
{
	if(v)
	{
		ow_port_out|=BIT_OUT_LIGHT;
	}
	else
	{
		ow_port_out&=~BIT_OUT_LIGHT;
	}
}

void hw_set_flash(bool v)
{
	if(v)
	{
		digitalWrite(4, HIGH);
		delay(100);
	}
	else
	{
		digitalWrite(4, LOW);
	}
}

void hw_do_vflash(bool v)
{
	if(v && work_do_flash_video)
	{
		hw_set_flash(v);
	}
	if(!v)
	{
		hw_set_flash(v);
	}
}

void hw_do_flash(bool v)
{
	if(v && work_do_flash_photo)
	{
		hw_set_flash(v);
	}
	if(!v)
	{
		hw_set_flash(v);
	}
}

void hw_tgl_led(void)
{
	ow_port_out^=BIT_OUT_LED;
}

void hw_do_led(bool v)
{
	if(v)
	{
		ow_port_out|=BIT_OUT_LED;
	}
	else
	{
		ow_port_out&=~BIT_OUT_LED;
	}
}

void hw_play_snd(int s)
{
	Log.printf("Play Sound %d\n\r",s);
	dfp_play(s);
}

void hw_snd_vol(uint8_t s)
{
	dfp_setVolume(s);
}

void hw_led_times(uint8_t t)
{
	Led_timer.setRepeats(t*2);
}

void hw_led_flush(void)
{
	led_count=0;
	Led_timer.setInterval(400);
	hw_led_times(cfg_global.switch_led_times);
	Led_timer.start();
}

void hw_cfg_load(void)
{
	hw_snd_vol(cfg_global.snd_vol);
	hw_led_times(cfg_global.switch_led_times);
	if(cfg_global.switch_led_on)
	{
		hw_do_led(true);
	}
	else
	{
		hw_do_led(false);
	}
}

void hw_init(void)
{
	// setup stuff
	// start Timer
	Dark_timer.setInterval(501);
	Dark_timer.expiredHandler(Dark_tf);
	Dark_timer.start();

	Relay_timer.setInterval(2000);
	Relay_timer.expiredHandler(Relay_tf);
	Relay_timer.setRepeats(1);

	Seconds_timer.setInterval(1000);
	Seconds_timer.expiredHandler(Seconds_tf);
	Seconds_timer.start();

	Led_timer.setInterval(400);
	Led_timer.expiredHandler(Led_tf);

	Ring_timer.setInterval(2000);
	Ring_timer.expiredHandler(Ring_tf);
	Ring_timer.setRepeats(1);

	Motion_timer.setInterval(2000);
	Motion_timer.expiredHandler(Motion_tf);
	Motion_timer.setRepeats(1);

	/**************************************************************************
	 *  DFPlayer
	 * ************************************************************************/
	Serial.flush();
	delay(20);

//	myDFPlayer.begin(Serial);
//	myDFPlayer.readType();
	dfp_init();
	hw_cfg_load();
}

void hw_loop(void)
{
	// loop stuff
	Relay_timer.run();
	Dark_timer.run();
	Seconds_timer.run();
	Led_timer.run();
	Motion_timer.run();
	Ring_timer.run();
	dfp_loop();
}

#ifdef aaa
#endif
